clear
clear matrix
set more off
set mem 10g

**This script is used to construct the student sample and make the appendix tables.**
**Written on 07/21/2010 WH

cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
capture log close
log using student_cohort, replace
tempfile sample
foreach year in 0304 0405 0506 0607 {
    cd /afs/econ.duke.edu/data/aa88/projects/nyc/data/`year'/transformed/stata_raw
    *************************************
    **Step 1: keep students in ags.dta.**
    *************************************
    tempfile ags
    use ags, clear
    keep osis_id composite_std_score
    ren composite_std_score ags_score
    egen std_ags=std(ags_score)
    gen from_ags=1 
    isid osis_id,sort
    save `ags',replace
   
    **Excluding duplicate observations.**
    **no such observations.*
    **Excluding applicants from private schools
    **Do this exclusion later.*
    **************************************************************************
    **Step 2: merge with app_choice.dta and exclude students not in cycle 1.**
    **************************************************************************
    if `year'==0304 {
          use matchdataR1,clear
    }
    else {
          use app_choice, clear
          keep if cycle==1
    }
    drop if program_code==""
    keep osis_id
    duplicates drop
    gen from_round1=1
    isid osis_id, sort
    merge osis_id using `ags', unique
    drop if _merge==1
    drop _merge
    isid osis_id,sort
    save `ags', replace

    ************************************************************************
    **Step 3: merge with ags_status.dat, and exclude those not matched.*****
    ************************************************************************
    use ags_status,clear
    keep osis_id program_code ags_status_cd
    keep if program_code=="K89S"|program_code=="X89S"|program_code=="M89S"
    gen from_ags_status=1
    sort osis_id
    merge osis_id using `ags', uniqusing
    keep if _merge==3
    drop _merge
    gen ch=(program_code!="")
    gen Offer=(ags_status_cd=="Y"|ags_status_cd=="A")

    **Within each program, rank students by ags score from highest to lowest.**
    bysort program_code: egen rank=rank(ags_score),field
    bysort program_code: egen napps=max(rank)
    bysort program_code: egen cutoffrank=max(rank*Offer)
    gen dist_cutoff=100*(cutoffrank-rank)/(napps-1)
    drop rank napps cutoffrank ags_status_cd
	
	**List the actual score and pctiles among all SHSAT test takers of each exam school's cutoff.**
	preserve
	gen cut=ags_score if dist_cutoff==0
	bysort program_code: egen cutoff=max(cut)
	keep osis_id ags_score program_code cutoff
	reshape wide cutoff,i(osis_id) j(program_code) string
	egen overall_rank=rank(ags_score),track
	gen pct=overall_rank/_N
	foreach sch in K89S X89S M89S{
	    gen pct`sch'=pct if cutoff`sch'==ags_score
		sort pct`sch'
		replace pct`sch'=pct`sch'[_n-1] if pct`sch'==.
		sort cutoff`sch'
		replace cutoff`sch'=cutoff`sch'[_n-1] if cutoff`sch'==.	
	}
	keep cutoff* pctK89S pctX89S pctM89S
	duplicates drop
	disp "cutoffs in the year:" `year'
	list
	restore

    reshape wide ch Offer dist_cutoff, i(osis_id) j(program_code) string
    **get the list of schools a student applied for**
    gen appshs=""
    foreach y of varlist ch*{
        replace appshs=appshs+","+substr("`y'",3,4) if `y'==1
    }
    replace appshs=substr(appshs,2,.)
    isid osis_id,sort
    merge osis_id using `ags', unique
    drop if _merge==1
    drop _merge
    isid osis_id,sort
    save `ags',replace
    ******************************************************************
    **Step 4: merge with student.dta, and exclude those not matched.**
    ******************************************************************
    use student,clear
    keep osis_id grade
    replace osis_id=trim(osis_id)
    gen from_student=1 
    capture destring grade, replace
    isid osis_id,sort
    merge osis_id using `ags', unique
    drop if _merge==1
    drop _merge
    isid osis_id,sort
    gen appyr="20"+substr("`year'",3,2)
    gen private=(substr(osis_id,1,1)=="A")
    save `ags',replace
    if "`year'"!="0304"{
        append using `sample'
    }
    save `sample',replace
}

**********************
**Appendix Table D1.**
**********************
use `sample',clear
keep if from_ags==1
tab appyr 
tab appyr if private==0
tab appyr if private==0&from_round1==1
tab appyr if private==0&from_round1==1&from_ags_status==1
keep if private==0&from_round1==1&from_ags_status==1
tab appyr if from_student==1
tab appyr if from_student==1&grade==8

**********************
**Appendix Table D2.**
**********************
**Indicator of students got an offer from at least one Specialied HS
gen Offer=(OfferK89S==1|OfferX89S==1|OfferM89S==1)
replace from_student=0 if from_student==.
table appyr Offer, c(mean from_student)
table appyr, c(mean from_student)
table Offer, c(mean from_student)
sum from_student
keep if from_ags==1&private==0&from_round1==1&from_ags_status==1&from_student==1&grade==8

**Exclude students who took the AGS tests in previous years.**
local a=1
bysort osis_id: egen ncount=sum(`a')
tab ncount
destring appyr,replace
bysort osis_id: egen mincohort=min(appyr)
keep if appyr==mincohort
tostring appyr,replace
drop from* ncount mincohort
tab appyr

cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
saveold student_cohort,replace

********************************************************************************
****######Part 2: Merge with student demographics and baseline scores.######****
********************************************************************************
tempfile sample
foreach year in 0304 0405 0506 0607 {
    ***************************************
    *****keep students in student.dta.*****
    ***************************************
    tempfile tmp
    cd /afs/econ.duke.edu/data/aa88/projects/nyc/data/`year'/transformed/stata_raw
    use student,clear
    drop if substr(osis_id,1,1)=="A"
    capture confirm variable std_id
    if !_rc {
           keep osis_id std_id date_of_birth home_lang_cd gender_cd ethnic_cd borough_cd /// 
		        zip_code two_percent_yn category_cd school_dbn
    }
    else {
           keep osis_id date_of_birth home_lang_cd gender_cd ethnic_cd borough_cd zip_code ///
		        two_percent_yn category_cd school_dbn
    }  
	ren school_dbn ms_dbn
    gen appyr="200"+substr("`year'",4,1)     
    gen from_student=1
    isid osis_id,sort  
    save `tmp', replace
    
    *************************************************************
    *****merge on student_lep, student_spec_ed, student_test*****
    *************************************************************
    tempfile lep spec_ed math_test 
    if `year'==0203 |`year'==0304 {
          use `tmp',clear
          sort std_id
          save `tmp',replace

          use student_lep,clear
          if `year'==0203 {
               keep std_id lep_status_cd
          }
          else {
               keep std_id reading_score listening_score lep_status_cd
          }
          gen from_lep=1
          isid std_id,sort
          save `lep',replace
          
          use student_spec_ed,clear
          if `year'==0203 {
              keep std_id
          }
          else{
              keep std_id spec_ed_yn
          }
          gen from_spec_ed=1
          isid std_id,sort
          save `spec_ed',replace
              
          use student_test,clear          
          keep test_id std_id test_type_cd test_cd scale_score raw_score 
          duplicates drop
          **There are multiple observations on the same test_type_cd within std_id.
          sort std_id test_type_cd test_id
          gen maxid=test_id
          replace maxid=. if std_id==std_id[_n+1]&test_type_cd==test_type_cd[_n+1]&test_id<test_id[_n+1]
          drop if maxid==.
          drop test_id maxid
          reshape wide test_cd scale_score raw_score, i(std_id) j(test_type_cd) string

          renames test_cdMATH scale_scoreMATH raw_scoreMATH\math_test_cd math_scale_score math_raw_score
          renames test_cdREADING scale_scoreREADING raw_scoreREADING\read_test_cd read_scale_score read_raw_score
          gen from_test=1
          isid std_id,sort
          merge std_id using `tmp' `lep' `spec_ed'            
    }
    else {
          use student_lep,clear
          keep osis_id reading_score listening_score lep_status_cd
          gen from_lep=1
          isid osis_id,sort
          save `lep',replace

          use student_spec_ed,clear
          keep osis_id spec_ed_yn 
          gen from_spec_ed=1
          isid osis_id,sort
          save `spec_ed',replace

          use student_math_test,clear 
          keep osis_id test_cd scale_score raw_score 
          renames test_cd scale_score raw_score \ math_test_cd math_scale_score math_raw_score
          gen from_math_test=1
          isid osis_id,sort
          save `math_test',replace

          use student_read_test,clear
          keep osis_id test_cd scale_score raw_score
          renames test_cd scale_score raw_score \ read_test_cd read_scale_score read_raw_score
          gen from_read_test=1
          isid osis_id,sort
          merge osis_id using `tmp' `lep' `spec_ed' `math_test'
    }
    drop _merge* _merge
    keep if from_student==1
    isid osis_id,sort
    save `tmp',replace
    
    capture destring ethnic_cd,replace
    if "`year'"!="0304"{
        append using `sample'
    }
    save `sample',replace
}
********************
**Recode variables**
********************
**convert home_lang to english vs. non-english
replace home_lang_cd="English" if home_lang_cd==""|home_lang_cd=="NO"
replace home_lang_cd="NON-ENGLISH" if home_lang_cd!="English"
    
**Compute age**
replace date_of_birth="0"+date_of_birth if substr(date_of_birth,2,1)=="-"
gen byear="19"+substr(date_of_birth,8,2) if substr(date_of_birth,8,1)=="8"|substr(date_of_birth,8,1)=="9"
gen bmon=substr(date_of_birth,4,3)
replace bmon="1" if bmon=="Jan"|bmon=="JAN"
replace bmon="2" if bmon=="Feb"|bmon=="FEB"
replace bmon="3" if bmon=="Mar"|bmon=="MAR"
replace bmon="4" if bmon=="Apr"|bmon=="APR"
replace bmon="5" if bmon=="May"|bmon=="MAY"
replace bmon="6" if bmon=="Jun"|bmon=="JUN"
replace bmon="7" if bmon=="Jul"|bmon=="JUL"
replace bmon="8" if bmon=="Aug"|bmon=="AUG"
replace bmon="9" if bmon=="Sep"|bmon=="SEP"
replace bmon="10" if bmon=="Oct"|bmon=="OCT"
replace bmon="11" if bmon=="Nov"|bmon=="NOV"
replace bmon="12" if bmon=="Dec"|bmon=="DEC"
destring byear bmon, replace
gen thisyr=appyr
destring thisyr, replace
gen age=thisyr-1-byear
replace age=age+(9-bmon)/12
replace age=round(age, .1)
replace age=. if date_of_birth==""|date_of_birth=="00-000-00"|age>21|age<10  
drop byear bmon thisyr
       
**Recode the categories of race
capture destring ethnic_cd, replace
replace ethnic_cd=1 if ethnic_cd==6 |ethnic_cd==.
la def race 1 "Other/Unknown" 2 "Asian" 3 "Hispanic" 4 "Black" 5 "White"
la var ethnic_cd race
gen black=(ethnic_cd==4)
gen hispanic=(ethnic_cd==3)
gen asian=(ethnic_cd==2)
gen white=(ethnic_cd==5)
gen other=(ethnic_cd==1)
    
**update the borough code by zip code
tempfile tmp
sort zip_code
save `tmp',replace   
cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/demand_project
use NYC_zip_borough,clear   
tostring zip_code, replace
isid zip_code,sort
merge zip_code using `tmp', uniqmaster replace update
drop if _merge==1
drop _merge
    
**Recode the categories of borough
gen manhattan=(borough_cd=="M")
gen brooklyn=(borough_cd=="K")
gen queen=(borough_cd=="Q")
gen bronx=(borough_cd=="X")
gen staten_island=(borough_cd=="R")
        
**Recode the variable gender_cd
gen female=1 if gender_cd=="F"
replace female=0 if gender_cd=="M"
        
**Recode the variable spec_ed_yn
gen sped=(spec_ed_yn=="Y")
         
**Recode the variable lep_status_cd 
gen lep_yn=1 if lep_status_cd=="Y"|lep_status_cd=="S"|lep_status_cd=="Z"|lep_status_cd=="A"
replace lep_yn=0 if lep_status_cd=="X"|lep_status_cd=="N"|lep_status_cd=="B"|lep_status_cd=="E" ///
                    |lep_status_cd=="P"|lep_status_cd==""

**Recode the variable category_cd
gen high=(category_cd=="HIGH")
gen middle=(category_cd=="MIDDLE")
gen low=(category_cd=="LOW")

**Standardize the baseline scores by application year.**
foreach x in math read{
    bysort appyr: egen mean`x'=mean(`x'_scale_score)
    bysort appyr: egen sd`x'=sd(`x'_scale_score)
    gen std_`x'_score=(`x'_scale_score-mean`x')/sd`x'
}
drop meanmath sdmath meanread sdread
*For students who appear twice in 2 consecutive years, keep their first year records.*
sort osis_id appyr
bysort osis_id: gen ncount=_n
keep if ncount==1
drop ncount 
isid osis_id,sort
save `sample',replace

tempfile student
keep osis_id appyr
isid osis_id,sort
save `student',replace
*******************************************
**merge on the free or priced lunch data.**
*******************************************
cd /afs/econ.duke.edu/data/aa88/projects/nyc/data/DOE/stata_version
use frpl,clear
keep if time_stamp=="0405"|time_stamp=="0506"
sort osis_id
merge osis_id using `student', uniqusing
drop if _merge==1
drop _merge
drop if (appyr=="2004"|appyr=="2005"|appyr=="2006")&time_stamp=="0506"
drop if appyr=="2007" & time_stamp=="0405"
isid osis_id,sort
merge osis_id using `sample',unique
drop if _merge==1
drop _merge
isid osis_id,sort
save `sample',replace

*******************************************
**merge on the 8th grade baseline scores.**
*******************************************
cd /afs/econ.duke.edu/data/aa88/projects/nyc/data/DOE/stata_version
use grade_8_exams_rv,clear
destring scale_score,replace
**Standardize the baseline scores by application year.**
bysort test_year test_cde: egen mean=mean(scale_score)
bysort test_year test_cde: egen sd=sd(scale_score)
gen std_score=(scale_score-mean)/sd
keep osis_id test_year test_cde std_score
ren std_score grade8_score
reshape wide grade8_score test_year, i(osis_id) j(test_cde) string
isid osis_id,sort
merge osis_id using `sample', unique
drop if _merge==1
drop _merge
isid osis_id,sort
save `sample',replace

*****************************************
**get enrollment info from audit files.**
*****************************************
cd /afs/econ.duke.edu/data/aa88/projects/nyc/data/DOE/stata_version
tempfile tmpaudit
foreach yr in 045 056 067 078{
     use audit`yr'_rv,clear
     drop id_key
     gen audityr="200"+substr("`yr'",3,1)
     duplicates drop
     isid osis_id,sort
     merge osis_id using `student',unique
     keep if _merge==3
     drop _merge
     if "`yr'"!="045" {
        append using `tmpaudit'
      }
     save `tmpaudit',replace
}
          
**convert bds to school_dbn**
replace school_dbn=substr(bds,2,2)+"M"+substr(bds,4,3) if school_dbn==""&substr(bds,1,1)=="1"
replace school_dbn=substr(bds,2,2)+"X"+substr(bds,4,3) if school_dbn==""&substr(bds,1,1)=="2"
replace school_dbn=substr(bds,2,2)+"K"+substr(bds,4,3) if school_dbn==""&substr(bds,1,1)=="3"
replace school_dbn=substr(bds,2,2)+"Q"+substr(bds,4,3) if school_dbn==""&substr(bds,1,1)=="4"
replace school_dbn=substr(bds,2,2)+"R"+substr(bds,4,3) if school_dbn==""&substr(bds,1,1)=="5"
drop bds
save `tmpaudit',replace
       
**using the new file "register_2009osis" to get the audit information in the year 2008.
use "/afs/econ.duke.edu/data/aa88/projects/nyc/data/DOE/stata_version/register_2009osis",clear
keep osis_id dbn grade_level
ren grade_level grade
ren dbn school_dbn
sort osis_id
merge osis_id using `student', uniqusing
keep if _merge==3
drop _merge
gen audityr="2009"
sort school_dbn
append using `tmpaudit'             
destring audityr appyr, replace
keep if audityr>appyr
tostring audityr appyr, replace
sort school_dbn appyr
save `tmpaudit',replace

**merge with the program.dta to identify screened programs.**
tempfile pgm
foreach year in 0304 0405 0506 0607{
    cd /afs/econ.duke.edu/data/aa88/projects/nyc/data/`year'/transformed/stata_raw
    if `year'==0304{
        tempfile tmpdbn
        use program,clear
        keep program_code program_type_cd hs_key
        sort hs_key
        save `tmpdbn',replace
  
        use school,clear
        keep hs_key school_dbn
        drop if hs_key==""|school_dbn==""
        isid hs_key,sort
        merge hs_key using `tmpdbn',uniqmaster
        keep if _merge==3
        drop _merge
    }
    else{
         use program,clear
    }
    keep program_code program_type_cd school_dbn
    gen keep=(lower(program_type_cd)=="s")
    bysort school_dbn: egen screened=sum(keep)
    replace screened=(screened>=1&screened!=.)
    keep school_dbn screened
    duplicates drop
    gen appyr="200"+substr("`year'",4,1)
    if `year'!=0304{
        append using `pgm'
    }
    save `pgm',replace
}
isid school_dbn appyr,sort
merge school_dbn appyr using `tmpaudit',uniqmaster
drop if _merge==1
drop _merge
     
**generate program_code for the specialized HS**
gen program_code="M89S" if school_dbn=="02M475" |school_dbn=="71M475"
replace program_code="X89S" if school_dbn=="10X445"|school_dbn=="72X445"
replace program_code="K89S" if school_dbn=="13K430"|school_dbn=="76K430"
replace program_code="NA"  if program_code==""
          
**Reshape the audit information across years.**        
gen at_shs=(program_code=="M89S"|program_code=="X89S"|program_code=="K89S")
gen atM89S=(program_code=="M89S")
gen atX89S=(program_code=="X89S")
gen atK89S=(program_code=="K89S")
gen schoolid=substr(school_dbn,3,4)
gen at_allexam=(at_shs==1|schoolid=="K449"|schoolid=="M692"|schoolid=="X696"|schoolid=="Q687"|schoolid=="R605"|schoolid=="M485")
drop schoolid
reshape wide program_code at_shs atM89S atX89S atK89S at_allexam school_dbn grade screened, i(osis_id) j(audityr) string 
isid osis_id,sort
merge osis_id using `sample',unique
drop if _merge==1
drop _merge
destring appyr,replace
*isid osis_id appyr,sort
isid osis_id,sort
save `sample',replace

cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
use student_cohort, clear
capture drop grade
capture drop appyr
*destring appyr,replace
*isid osis_id appyr,sort
*merge osis_id appyr using `sample', unique
isid osis_id,sort
merge osis_id using `sample',unique
gen app_sample=(_merge==1|_merge==3)
drop _merge
save `sample',replace

**Define grade9 enrollment indicators and offer status**
destring appyr,replace
gen grade9_pgm=""
gen grade9_dbn=""
gen grade9_screened=.
forvalues appyr=2004/2007{
    local yr=`appyr'+1
    replace grade9_pgm=program_code`yr' if appyr==`appyr'
    replace grade9_dbn=school_dbn`yr' if appyr==`appyr'
    replace grade9_screened=screened`yr' if appyr==`appyr'
}  
gen EnrollmentX89S=(grade9_pgm=="X89S")
gen EnrollmentM89S=(grade9_pgm=="M89S")
gen EnrollmentK89S=(grade9_pgm=="K89S")
gen Enrollment=(EnrollmentX89S==1|EnrollmentM89S==1|EnrollmentK89S==1)
gen grade9_id=substr(grade9_dbn,3,4)
gen grade9_Other=(grade9_id=="K449"|grade9_id=="M692"|grade9_id=="X696"|grade9_id=="Q687"|grade9_id=="R605"|grade9_id=="M485")
drop grade9_id

gen grade9_inexam=0
gen grade9_inNYC=0
destring appyr,replace
forvalues audityr=2005/2009{
	replace grade9_inexam=1 if at_shs`audityr'==1 & `audityr'==appyr+1
	replace grade9_inNYC=1 if school_dbn`audityr'!="" & `audityr'==appyr+1
}

**Compute the peer mean of 8th grade English and Math scores.**
bysort grade9_dbn: egen peer_Math=mean(grade8_scoreSTM)
bysort grade9_dbn: egen peer_English=mean(grade8_scoreELA)
bysort grade9_dbn: egen peer_fem=mean(female)
bysort grade9_dbn: egen peer_frpl=mean(frpl)
bysort grade9_dbn: egen peer_blkhisp=mean(black==1|hispanic==1)
sort grade9_dbn
save `sample',replace

use "/afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes/teacher",clear
keep school_dbn year num_teach per_valid_cert per_fewer_3yrs_exp per_mas_plus
destring year,replace
keep if year>2004
collapse (mean) num_teach per_valid_cert per_fewer_3yrs_exp per_mas_plus,by(school_dbn)
ren school_dbn grade9_dbn
isid grade9_dbn,sort
merge grade9_dbn using `sample',uniqmaster
tab _merge
drop if _merge==1
drop _merge
gen grade9_schoolid=substr(grade9_dbn,3,4)
sort grade9_schoolid
save `sample',replace

use "/afs/econ.duke.edu/data/aa88/projects/nyc/results/small_schools/weiwei/class_size",clear
collapse (mean) avg_class*, by(schoolid)
ren schoolid grade9_schoolid
merge grade9_schoolid using `sample', uniqmaster
drop if _merge==1
drop _merge

compress
cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
saveold student_sample, replace

*******************************************************************
****########Part 3: Merge with the regents test scores.########****
*******************************************************************
**Step 1: get the student sample.**
tempfile tmp
use student_sample,clear 
destring appyr,replace   
tab appyr if app_sample==1,miss

*keep if app_sample==1     
gen id=1
expand 8
*bysort osis_id appyr: gen subj=sum(id)
bysort osis_id: gen subj=sum(id)
gen coursecd="EXRL" if subj==1
replace coursecd="HXR$" if subj==2
replace coursecd="HXRA" if subj==3
replace coursecd="MXRA" if subj==4
replace coursecd="MXRB" if subj==5
replace coursecd="MathAE" if subj==6
replace coursecd="MathBG" if subj==7
replace coursecd="SXRK" if subj==8
/*
gen courseid=1 if coursecd=="MathAE"
replace courseid=2 if coursecd=="MathBG"
replace courseid=3 if coursecd=="EXRL"
replace courseid=4 if coursecd=="HXR$"
replace courseid=5 if coursecd=="HXRA"
replace courseid=6 if coursecd=="SXRK"
**count number of applicants.**
tabulate courseid appyr if app_sample==1,matcell(freq1) matrow(course1) matcol(year)

**If a student shows up in multiple years, just keep the first year record.**
*We have some students who did not take exam test before, but then take the SHSAS in later years.
sort osis_id coursecd appyr
bysort osis_id coursecd: egen minyr=min(appyr)
keep if appyr==minyr
drop minyr
*/
drop id subj 
isid osis_id coursecd,sort
save `tmp',replace
      
tempfile tmp1
keep osis_id appyr app_sample
duplicates drop
isid osis_id,sort
save `tmp1', replace

**Step 2: get regent scores and figure out the "termcd" in the regent file**
*Note: This part is used to figure out the meaning of termcd in the regents test files.
**The rules used in the following script is based on Bucher Scott's email (Oct 6, 2009) forwarded 
**by Parag (Nov 16, 2009).
tempfile regent
use "/afs/econ.duke.edu/data/aa88/projects/nyc/data/DOE/stata_version/regents_rct_rv",clear
**keep scores with numeric values
replace mark="" if regexm(mark, "[0-9]$")==0
keep if regexm(mark, "[0-9]$")==1
destring mark, replace     
gen term="f" if termcd=="1" |termcd=="5"|termcd=="a"
replace term="s" if termcd=="2" |termcd=="3"|termcd=="4"|termcd=="6"|termcd=="7"
gen rawcourse=coursecd

#delimit ;
local dbn 79M573 79M612 32K564 02M560 10X319 02M575 22K585 12X480 03M505 02M570 
          21K525 21K540 19K409 17K489 15K698 14K454 14K640 07X379 11X265 15K529 
          08X377 05M285 21K728 02M303 25Q792 18K578 24Q520 19K431;
#delimit cr
foreach x of local dbn{
    replace term="f" if termcd=="2" & schooldbn=="`x'"
}
destring schoolyear,replace
foreach x in EXRL2 MXRA2 MXRB2 MXRE2 HXR$2 HXRA2 SXRC2 SXR$2 SXRE2 SXRK2{
    replace term="s" if coursecd=="`x'"
    replace schoolyear=schoolyear+1 if coursecd=="`x'"
    replace coursecd=substr("`x'",1,4) if coursecd=="`x'"
}
tostring schoolyear,replace
drop if schoolyear=="2008"
save `regent',replace
 
**Append the 2008 regents score information to the old regent files.
use "/afs/econ.duke.edu/data/aa88/projects/nyc/data/DOE/stata_version/credit_regent_2009osis.dta", clear
foreach x of varlist reg*{
    local name=substr("`x'",1,3)+substr("`x'",5,4)+substr("`x'",10,5)
    ren `x' `name'
}
reshape long reg, i(osis_id) j(coursecd) string
gen term=substr(coursecd, 5,1)
gen schoolyear="2008" 
ren reg mark 
drop if mark==.
replace coursecd="EXRL" if substr(coursecd,1,4)=="engl"
replace coursecd="HXR$" if substr(coursecd,1,4)=="glob"
replace coursecd="HXRA" if substr(coursecd,1,4)=="hist"
replace coursecd="MXRA" if substr(coursecd,1,4)=="mtha"
replace coursecd="MXRB" if substr(coursecd,1,4)=="mthb"
replace coursecd="SXR$" if substr(coursecd,1,4)=="phys"
replace coursecd="SXRK" if substr(coursecd,1,4)=="livn"
replace coursecd="SXRU" if substr(coursecd,1,4)=="esci"
replace coursecd="SXRX" if substr(coursecd,1,4)=="chem"
replace coursecd="MXRE" if substr(coursecd,1,4)=="inta"
replace coursecd="MXRG" if substr(coursecd,1,4)=="geom"
append using `regent'
save `regent',replace     

**Step 3: keep students' first numeric regent score, standardize the score by subject+test date**
*Note: keep the first time score after students enrolled at school.
*Drop student who take a test in middle school.
use `regent', clear   
#delimit ;
keep if coursecd=="EXRL"|coursecd=="MXRA"|coursecd=="MXRB"|coursecd=="HXR$"
        |coursecd=="HXRA"|coursecd=="SXRK"|coursecd=="MXRE"|coursecd=="MXRG";
#delimit cr 
keep osis_id coursecd schoolyear termcd mark term schooldbn rawcourse 
ren schooldbn testyr_dbn
drop if osis_id==""|mark==.
sort osis_id
merge osis_id using `tmp1',uniqusing
keep if _merge==3
tab _merge appyr
drop _merge
tab coursecd appyr
save `regent',replace

**Construct a math variable which is the "first math test taken after starting 9th grade"
**For the Math test, show the outcome analysis for both MXRA and our constructed math**
gen n=1
replace n=2 if coursecd=="MXRA"
replace n=2 if coursecd=="MXRB"
expandcl n, generate(newcl) cluster(coursecd)
gen course=coursecd
replace coursecd="MathAE" if (course=="MXRA"&newcl==5)|course=="MXRE"
replace coursecd="MathBG" if (course=="MXRB"& newcl==7)|course=="MXRG"
drop n newcl 

**generate test dates.**
gen test_date=schoolyear+termcd
replace test_date=schoolyear+term if schoolyear=="2008"

**For MathAE, take MXRA if a student has both MXRA and MXRE on the same test dates.**
**For MathBG, take MXRB if a student has both MXRB and MXRG on the same test dates.**
gen testA=(course=="MXRA")
gen testE=(course=="MXRE")
bysort osis_id coursecd test_date: egen isA=sum(testA)
bysort osis_id coursecd test_date: egen isE=sum(testE)
gen bothAE=(isA>=1&isE>=1)
drop if coursecd=="MathAE" & course=="MXRE" &bothAE==1
drop testA testE isA isE bothAE
gen testB=(course=="MXRB")
gen testG=(course=="MXRG")
bysort osis_id coursecd test_date: egen isB=sum(testB)
bysort osis_id coursecd test_date: egen isG=sum(testG)
gen bothBG=(isB>=1&isG>=1)
drop if coursecd=="MathBG" & course=="MXRG" &bothBG==1
drop testB testG isB isG bothBG

**keep test subjects needed for regression analysis.**
gen courseid=1 if coursecd=="MathAE"
replace courseid=2 if coursecd=="MathBG"
replace courseid=3 if coursecd=="EXRL"
replace courseid=4 if coursecd=="HXR$"
replace courseid=5 if coursecd=="HXRA"
replace courseid=6 if coursecd=="SXRK"
keep if courseid==1|courseid==2|courseid==3|courseid==4|courseid==5|courseid==6

**Exclude students who take regent tests in middle school.
destring schoolyear,replace
gen bf_enroll=1 if schoolyear<appyr &appyr!=.
bysort coursecd osis_id: egen test_bf_enroll=total(bf_enroll)
bysort coursecd osis_id: gen keep=(_n==1)
**Count number of students with scores observed before treatment.**
tabulate courseid appyr if keep==1 & test_bf_enroll>=1&app_sample==1,matcell(freq2) matrow(course2) matcol(year)
drop if test_bf_enroll>=1
tostring schoolyear, replace
**count number of students with scores observed after treatment.**
tabulate courseid appyr if keep==1&app_sample==1,matcell(freq3) matrow(course3) 
drop bf_enroll test_bf_enroll 

**count number of students with multiple scores observed after treatment.**
local a=1
bysort coursecd osis_id: egen num_score=sum(`a')
tab coursecd appyr if keep==1&num_score>1&app_sample==1

**count number of students with multiple different scores observed after treatment.**
bysort coursecd osis_id: egen minscore=min(mark)
bysort coursecd osis_id: egen maxscore=max(mark)
gen diff=(minscore!=maxscore)
tabulate courseid appyr if keep==1&num_score>1&diff==1&app_sample==1,matcell(freq5) matrow(course5)
drop minscore maxscore num_score keep diff

**keep the first time score after students enrolled at school.
sort test_date
egen testdateid=group(test_date)
bysort coursecd osis_id: egen mindate=min(testdateid)
keep if testdateid==mindate
drop mindate 

**count number of students with multiple scores observed after treatment, on first date.**
local a=1
bysort coursecd osis_id: egen num_score=sum(`a')
bysort coursecd osis_id: gen keep=(_n==1)
tab coursecd appyr if keep==1&num_score>1&app_sample==1

**count number of students with multiple different scores observed after treatment, on first date.**
bysort coursecd osis_id: egen minscore=min(mark)
bysort coursecd osis_id: egen maxscore=max(mark)
gen diff=(minscore!=maxscore)
tabulate courseid appyr if keep==1&num_score>1&diff==1&app_sample==1,matcell(freq7) matrow(course7)
preserve
keep if num_score>1&diff==1&app_sample==1
keep coursecd appyr osis_id mark schoolyear termcd term testyr_dbn course rawcourse
drop if coursecd=="Math"|coursecd=="MXRA"
outsheet using "/afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes/diff_multi_scores.csv",replace
restore

**Multiple different scores on same test dates.**
*If a student has tests ending with and without 2 on the same date, drop tests ending with 2.
*In all other cases, set the scores as missing.
drop if num_score>1&substr(rawcourse,5,1)=="2"
drop num_score minscore maxscore diff
local a=1
bysort coursecd osis_id: egen num_score=sum(`a')
bysort coursecd osis_id: egen minscore=min(mark)
bysort coursecd osis_id: egen maxscore=max(mark)
gen diff=(minscore!=maxscore)
replace mark=. if num_score>1 & diff==1
drop if mark==.
keep osis_id coursecd mark test_date schoolyear term appyr testyr_dbn app_sample testdateid courseid
duplicates drop

*If a student has the same test scores on the same test dates (schoolyear+termcd), keep one record.**
duplicates drop coursecd osis_id test_date mark, force
sort appyr coursecd
save `regent',replace

**get the "most common test dates.**
keep if app_sample==1
isid osis_id coursecd
table appyr test_date coursecd
bysort appyr coursecd test_date: egen num=total(mark!=.)
bysort appyr coursecd: egen maxnum=max(num)
gen common_date=testdateid if num==maxnum
keep appyr coursecd common_date courseid
keep if common_date!=.
duplicates drop
isid appyr coursecd,sort
merge appyr coursecd using `regent', uniqmaster
drop if _merge==1
drop _merge
gen on_common=1 if testdateid==common_date & test_date!=""
gen bf_common=1 if testdateid<common_date & common_date!=.
gen af_common=1 if testdateid>common_date &test_date!=""
disp "count number of students with scores observed before the most common date."
tabulate courseid appyr if on_common==1&app_sample==1,matcell(freq8) matrow(course8)
disp "count number of students with scores observed before the most common date."
tabulate courseid appyr if bf_common==1&app_sample==1,matcell(freq9) matrow(course9)
disp "count number of students with scores observed after the most common date."
tabulate courseid appyr if af_common==1&app_sample==1,matcell(freq10) matrow(course10)
sort coursecd schoolyear term
save `regent',replace

**Standardize test scores by test date and subject.
drop if coursecd==""
replace term=".m" if term==""
collapse (mean) avgmark=mark (sd) sdmark=mark, by(coursecd schoolyear term)
isid coursecd schoolyear term,sort
merge coursecd schoolyear term using `regent', uniqmaster
drop if _merge==1
drop _merge
gen std_mark=(mark-avgmark)/sdmark
drop avgmark sdmark
isid osis_id coursecd,sort
merge osis_id coursecd using `tmp', unique
drop if _merge==1
drop _merge
save `tmp',replace

**Step 4: Define the treatment variable Di: years in small schools.**
**Di=years in any exam school
**count how many days students stay in an exam hs since application date to the test time.**   
destring schoolyear, replace
gen testyear=schoolyear+1    
gen testdate=mdy(1,31, testyear) if term=="f"
replace testdate=mdy(6,1, testyear) if term=="s"|term==""
gen enrollbftest=mdy(9,01, schoolyear)
gen exam_hs_days=0
gen allexam_days=0 
forvalues audityr=2005/2009{
    local yr=`audityr'-1
    replace testyr_dbn=school_dbn`audityr' if schoolyear==`yr' &testyr_dbn==""
    replace exam_hs_days=exam_hs_days+testdate-enrollbftest if at_shs`audityr'==1&schoolyear==`yr'
    replace exam_hs_days=exam_hs_days+365 if at_shs`audityr'==1&(`yr'>appyr&`yr'<=schoolyear)
	replace allexam_days=allexam_days+testdate-enrollbftest if at_allexam`audityr'==1&schoolyear==`yr'
    replace allexam_days=allexam_days+365 if at_allexam`audityr'==1&(`yr'>appyr&`yr'<=schoolyear)
}

foreach x of varlist *_days{
    local pos=strpos("`x'","_days")
    local v=substr("`x'",1,`pos'-1)
    gen `v'_yrs=`v'_days/365
}

**Set scores as missing if the testyr_dbn is missing.**
replace std_mark=. if testyr_dbn==""
gen have_score=(std_mark!=.)
**count the number of students having post-assignment numeric scores for at least one subject.**
preserve
keep if have_score==1&app_sample==1
collapse (max) have_score, by(appyr osis_id)
tab appyr 
restore

********************************************
**Step 5: Define the discontinuity sample.**
********************************************
**generate above cutoff dummies.**
gen above_cutoffK89S=(dist_cutoffK89S>=0)
gen above_cutoffX89S=(dist_cutoffX89S>=0)
gen above_cutoffM89S=(dist_cutoffM89S>=0)

**generate interactions terms betweeen above_cutoff dummies and running variables.**
foreach sch in K89S X89S M89S{
    gen dist2_cutoff`sch'=dist_cutoff`sch'*dist_cutoff`sch'
    gen dist3_cutoff`sch'=dist_cutoff`sch'*dist2_cutoff`sch'
    gen intdist_cutoff`sch'=above_cutoff`sch'*dist_cutoff`sch'
    gen intdist2_cutoff`sch'=above_cutoff`sch'*dist2_cutoff`sch'
    gen intdist3_cutoff`sch'=above_cutoff`sch'*dist3_cutoff`sch'
}

**Define the discontinuity samples.**
local disc_lb=-10
local disc_ub=10
foreach sch in K89S X89S M89S{
    gen disc_samp_`sch'=(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub')
}      
drop testdate enrollbftest *days 
tostring schoolyear appyr,replace

***************************
**make Appendix table B4.**
***************************
foreach row in 2 3 5 7 8 9 10{
    forvalues subj=1/6{
	    capture confirm matrix panel`subj'
		if _rc{
		    *Generalize this line by count #values of appyr
		    matrix panel`subj'=[.,.,.,.]
		}
	    local c=course`row'[`subj',1]
	    if `c'!=`subj'{
		    *Generalize this line by count #values of appyr
		    matrix tmp=[0,0,0,0]
		}
		else{
            matrix tmp=freq`row'[`subj',1...]
		}
		matrix panel`subj'=(panel`subj'\tmp)
	}
}
matrix Stat=(panel1\panel2\panel3\panel4\panel5\panel6)
*Generalize this line by count #values of appyr
matrix col_tot=Stat[1...,1]+Stat[1...,2]+Stat[1...,3]+Stat[1...,4]
matrix Stat=(Stat,col_tot)
mat2txt, matrix(Stat) saving("tableD3.txt") replace

cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
compress
**keep variables needed for tables and plots.**
keep osis_id app_sample schoolyear grade9_dbn coursecd appyr std_mark mark ///
    dist*_cutoff* above_cutoff* intdist*_cutoff* chM89S chX89S chK89S appshs std_ags  ///
    female black asian hispanic other lep_yn sped frpl grade8_score* ///
    have_score testyr_dbn Enrollment* Offer* peer* *_yrs grade9* ///
    num_teach per_valid_cert per_fewer_3yrs_exp per_mas_plus avg_class* ///
    grade9_inNYC grade9_inexam

saveold regression_sample, replace

capture erase student_cohort.dta
capture erase student_sample.dta


log close
