clear
clear mata
clear matrix
set more off
set mem 4g
set matsize 800

*This script is used to do empirical analysis and generate NYC tables and plots reported in the paper.**
*Updated on 05/10/2013 WH

cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
capture log close
log using regression_log.txt,replace text
quietly: do "/afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes/rdob_mod2.ado"

********************************************************************
****Regression Discontinuity Estimator with Optimal Bandwidth ******
**Using IK program "rdob" downloaded from:
** http://www.economics.harvard.edu/faculty/imbens/software_imbens**
********************************************************************
tempfile tmp
use regression_sample,clear
keep if app_sample==1

destring schoolyear,replace
**generate peer-mean of regents scores after assignment.**
bysort grade9_dbn coursecd appyr: egen postmean=mean(std_mark) if std_mark!=.

drop if appyr=="2007"&(coursecd=="EXRL"|coursecd=="HXRA")

egen risk_set=group(appyr)
**Define the discontinuity samples.**
local disc_lb=-6
local disc_ub=6
capture drop disc_samp*
foreach sch in K89S X89S M89S{
    gen disc_samp_`sch'=(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub')
} 
gen disc_all=(disc_samp_K89S==1|disc_samp_X89S==1|disc_samp_M89S==1)     
local demos="female black asian hispanic other lep_yn sped frpl"
local baseline="grade8_scoreSTM grade8_scoreELA"
keep osis_id have_score testyr_dbn schoolyear std_mark coursecd above_cutoff* /// 
  dist*_cutoff* appshs appyr std_ags intdist*_cutoff* disc_* `demos' `baseline' ///
  Enrollment* risk_set chM89S chX89S chK89S peer* *_yrs mark grade9*  ///
   num_teach per_valid_cert per_fewer_3yrs_exp per_mas_plus 
xi i.schoolyear i.appyr    

**generate clustering id.**
egen clusterid=group(testyr_dbn schoolyear) 
replace coursecd="HXR" if coursecd=="HXR$" 

**high-end students.**   
gen baseline=grade8_scoreSTM if coursecd=="MXRA"|coursecd=="MXRB"|coursecd=="MathAE"|coursecd=="MathBG"|coursecd=="SXRK"
replace baseline=grade8_scoreELA if coursecd=="EXRL"|coursecd=="HXR"|coursecd=="HXRA"
bysort appyr coursecd: egen median=median(baseline)
bysort appyr coursecd: egen qntl=pctile(baseline),p(75)
gen above_half=(baseline>=median)
gen above_qntl=(baseline>=qntl)
gen blackhisp=(black==1|hispanic==1)
gen all=1
gen above85=(mark>=85) if std_mark!=.

**Mean follow-up rates in the attrition table.**
tabstat have_score if disc_all==1, by(coursecd) stat(mean n)

**Mean Test Scores. (Full sample).**
tabstat std_mark,by(coursecd) long col(stat)

**Mean Test Scores. (Discontinuity sample).**
tabstat std_mark if disc_all==1,by(coursecd) stat(mean n)

tempfile unstack
save `unstack',replace

**generate two other cutoff indicators.**
gen above_oth1K89S=above_cutoffX89S
gen above_oth2K89S=above_cutoffM89S
gen above_oth1X89S=above_cutoffK89S
gen above_oth2X89S=above_cutoffM89S
gen above_oth1M89S=above_cutoffK89S
gen above_oth2M89S=above_cutoffX89S

**Reshape data set.**
drop EnrollmentK89S EnrollmentX89S EnrollmentM89S
reshape long above_cutoff dist_cutoff dist2_cutoff dist3_cutoff intdist_cutoff ///
        intdist2_cutoff intdist3_cutoff disc_samp_ ch potential_yrs above_oth1 ///
		above_oth2,i(osis_id coursecd) j(group) string
*Students could appear in the dataset multiple times. #times=#exam schools he/she ranked.
keep if ch==1

**generate interaction terms between 2 other cutoff indicators and running variables.**
forvalues i=1/2{
    gen oth`i'_dist_cutoff=above_oth`i'*dist_cutoff
	gen oth`i'_dist2_cutoff=above_oth`i'*dist2_cutoff
	gen oth`i'_dist3_cutoff=above_oth`i'*dist3_cutoff	
}

**Pre-trim the NYC data to 6.**
keep if disc_samp_==1

save `tmp',replace

local options="cells(b(star fmt(%9.3f)) se(par)) starlevels(* 0.1 ** 0.05 *** 0.01) stats(N, fmt(%9.0g)) mlabels(none) collabels(none)"
************************************************************************
**##########table 8 and B2: Reduced Form, Attrition Results.##########**
************************************************************************
capture erase reduced.txt
capture erase attrition.txt
**Get the optimal bandwidth with the IK program**
foreach test in MathAE MathBG EXRL HXR HXRA SXRK{      
    use `tmp',clear
    keep if coursecd=="`test'"	
    foreach sch in K89S X89S M89S{
        **IK estimation for each type of exam schools.**
        local y std_mark
        local x dist_cutoff
        capture drop keep
        egen keep=rowmiss(`y' `x')
        qui rdob_mod2 `y' `x' if keep==0&group=="`sch'", c(0) dm(1)
		disp "Optimal Bandwidth for `test' of `sch': `r(h_opt)'"
        local h_opt=r(h_opt)
		
        **weight for local linear regression
        qui gen lambda`sch'=(abs(`x')<=`h_opt')*(1-abs(`x')/`h_opt') if group=="`sch'"
        **generate indicator=1, if a record is included in the IK estimation.**
        qui gen ik`sch'=(lambda`sch'!=0&lambda`sch'!=.)
		
        foreach reg in reduced attrition{
            if "`reg'"=="reduced"{
                local y std_mark
                local z="_Ischoolyea* _Iappyr*"
                local ses="cluster(clusterid)"
            }
            else if "`reg'"=="attrition"{
                local y have_score
                local z="_Iappyr*"
                local ses="robust"
            }  
			local running_controls="dist_cutoff intdist_cutoff"  
            eststo `reg'ik`sch': qui regress `y' above_cutoff `running_controls' `z' [aw=lambda`sch'] if group=="`sch'",`ses'
           
            **Parametric Estimation for each type of exam schools.**
            local running_controls="dist*_cutoff intdist*_cutoff"
            eststo `reg'para`sch':qui regress `y' above_cutoff `running_controls' `z' if disc_samp_==1&group=="`sch'",`ses'
        }	
    }

    **IK estimation for all exam schools.**
	replace ikK89S=0 if ikK89S==.
	replace ikX89S=0 if ikX89S==.
	replace ikM89S=0 if ikM89S==.
    gen ik_all=(ikK89S==1|ikX89S==1|ikM89S==1)
	gen lambda_all=lambdaK89S if ikK89S==1
	replace lambda_all=lambdaX89S if ikX89S==1
	replace lambda_all=lambdaM89S if ikM89S==1
    **get the stacked version of running variables and interactions.**
    qui foreach x in above_cutoff dist_cutoff intdist_cutoff{
        gen ikK89S_`x'=ikK89S*`x'
        gen ikX89S_`x'=ikX89S*`x'
        gen ikM89S_`x'=ikM89S*`x'
    }
	qui foreach sch in K89S X89S M89S{
        gen ik`sch'_testyr=ik`sch'*schoolyear
	    gen ik`sch'_risk=ik`sch'*risk_set
		gen disc`sch'=(disc_samp_==1&group=="`sch'")
	}
    **Parametric estimation for all exam schools.**
    **get the stacked version of running variables and interactions.**
    qui foreach x of varlist above_cutoff dist*_cutoff intdist*_cutoff{
        gen discK89S_`x'=discK89S*`x'
        gen discX89S_`x'=discX89S*`x'
        gen discM89S_`x'=discM89S*`x'
    }
	qui foreach sch in K89S X89S M89S{
        gen disc`sch'_testyr=disc`sch'*schoolyear
	    gen disc`sch'_risk=disc`sch'*risk_set
	}
	
    foreach reg in reduced attrition{
        if "`reg'"=="attrition"{
            local y have_score
            local z1="i.ikK89S_risk i.ikX89S_risk i.ikM89S_risk"
			local z2="i.discK89S_risk i.discX89S_risk i.discM89S_risk"
            local ses="cluster(osis_id)"
        }
		else {
			local z1="i.ikK89S_testyr i.ikX89S_testyr i.ikM89S_testyr i.ikK89S_risk i.ikX89S_risk i.ikM89S_risk"
			local z2="i.discK89S_testyr i.discX89S_testyr i.discM89S_testyr i.discK89S_risk i.discX89S_risk i.discM89S_risk"
            local ses="cluster(osis_id clusterid)"         
            if "`reg'"=="score8184"|"`reg'"=="score6164"{
		        local y `reg'
			}
			if "`reg'"=="reduced"{
                local y std_mark
			}
		}
        local running_controls="ikX89S ikM89S ik*_dist_cutoff ik*_intdist_cutoff"  
        eststo `reg'ikall: qui xi: ivreg2 `y' above_cutoff `running_controls' `z1' [aw=lambda_all],`ses'
		
        local running_controls="discX89S discM89S disc*_dist*_cutoff disc*_intdist*_cutoff" 
        eststo `reg'paraall: qui xi: ivreg2 `y' above_cutoff `running_controls' `z2' if disc_samp_==1,`ses'
		
        estout `reg'para*89S `reg'paraall `reg'ik*89S `reg'ikall using `reg'.txt,keep(above_cutoff) `options' append  			
	}  
} 
******************************************************************************************
**##########table B4: Reduced Form Estimates for High-End, black and hispanic.##########**
******************************************************************************************
**compute the baseline mean value for high-end students, black and hispanic students.**
**Baseline Mean for Upper Half and Upper Quartile.**
use `tmp',clear
tabstat std_mark above85 if above_half==1&(disc_all==1),by(coursecd) stat(mean n) 
tabstat std_mark above85 if above_qntl==1&(disc_all==1),by(coursecd) stat(mean n)
tabstat std_mark above85 if blackhisp==1&(disc_all==1),by(coursecd) stat(mean n)

est clear
scalar drop _all
capture erase subgroups.txt
**Get the optimal bandwidth with the IK program**
foreach test in MathAE MathBG EXRL HXR HXRA SXRK{ 
    foreach samp in all above_half above_qntl blackhisp{     
        use `tmp',clear
        keep if `samp'==1
        keep if coursecd=="`test'"       
        foreach sch in K89S X89S M89S{               
            **IK estimation for each type of exam schools.**
            local y std_mark
            local x dist_cutoff
            local running_controls="dist_cutoff intdist_cutoff"  
            capture drop keep
            egen keep=rowmiss(`y' `x')
            qui rdob_mod2 `y' `x' if keep==0&group=="`sch'", c(0) dm(1)
			disp "Optimal Bandwidth for `test' of `sch' in `samp': `r(h_opt)'"
            local h_opt=r(h_opt)
		    
            **weight for local linear regression
            qui gen lambda`sch'=(abs(`x')<=`h_opt')*(1-abs(`x')/`h_opt') if group=="`sch'"
            **generate indicator=1, if a record is included in the IK estimation.**
            qui gen ik`sch'=(lambda`sch'!=0&lambda`sch'!=.) if group=="`sch'"
        }
        **IK estimation for all exam schools.**
		capture drop lambda_all 
        replace ikK89S=0 if ikK89S==.
        replace ikX89S=0 if ikX89S==.
        replace ikM89S=0 if ikM89S==.
		gen lambda_all=lambdaK89S if ikK89S==1
		replace lambda_all=lambdaX89S if ikX89S==1
		replace lambda_all=lambdaM89S if ikM89S==1
        **get the stacked version of running variables and interactions.**
        qui foreach x in dist_cutoff intdist_cutoff{
            gen ikK89S_`x'=ikK89S*`x'
            gen ikX89S_`x'=ikX89S*`x'
            gen ikM89S_`x'=ikM89S*`x'
        }
		qui foreach sch in K89S X89S M89S{
            gen ik`sch'_testyr=ik`sch'*schoolyear
	        gen ik`sch'_risk=ik`sch'*risk_set
	    }
     
		local y="std_mark"
        local ses="cluster(osis_id clusterid)"           
        local running_controls="ikX89S ikM89S ik*_dist_cutoff ik*_intdist_cutoff" 
		local z="i.ikK89S_testyr i.ikX89S_testyr i.ikM89S_testyr i.ikK89S_risk i.ikX89S_risk i.ikM89S_risk"
        ren above_cutoff `test' 
        eststo `samp'_ik: qui xi: ivreg2 `y' `test' `running_controls' `z' [aw=lambda_all],`ses'    

    } 
    estout above_half_ik above_qntl_ik blackhisp_ik using subgroups.txt,keep(`test') `options' append  
}
 
*************************************************************
**##########table B3: Covariate Balance Analysis.##########**
*************************************************************
est clear
capture erase covbalance.txt
tempfile covariate
use regression_sample,clear
keep if app_sample==1
drop if appyr=="2007"&(coursecd=="EXRL"|coursecd=="HXRA")
*egen risk_set=group(appshs appyr)
egen risk_set=group(appyr)
**Define the discontinuity samples.**
local disc_lb=-6
local disc_ub=6
capture drop disc_samp*
foreach sch in K89S X89S M89S{
    gen disc_samp_`sch'=(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub')
} 
save `covariate',replace

use `covariate',clear
local outcome="female black hispanic frpl lep_yn"
keep osis_id `outcome' risk_set disc_samp* above_cutoff* dist*_cutoff* intdist*_cutoff*
duplicates drop
isid osis_id
tabstat `outcome' if disc_samp_K89S==1|disc_samp_X89S==1|disc_samp_M89S==1, stat(mean n)

**generate two other cutoff indicators.**
gen above_oth1K89S=above_cutoffX89S
gen above_oth2K89S=above_cutoffM89S
gen above_oth1X89S=above_cutoffK89S
gen above_oth2X89S=above_cutoffM89S
gen above_oth1M89S=above_cutoffK89S
gen above_oth2M89S=above_cutoffX89S

local options="cells(b(star fmt(%9.3f)) se(par)) starlevels(* 0.1 ** 0.05 *** 0.01) stats(F_value p_value N, fmt(%9.3f %9.3f %9.0g)) mlabels(none) collabels(none)"
************************************
**Reshape data to the long format.**
************************************
drop if female==.|black==.|hispanic==.|frpl==.|lep_yn==.
foreach var of varlist `outcome'{
    ren `var' y_`var' 
}
reshape long y_ , i(osis_id) j(covariate) string

reshape long above_cutoff dist_cutoff dist2_cutoff dist3_cutoff intdist_cutoff intdist2_cutoff intdist3_cutoff disc_samp_ above_oth1 above_oth2, i(osis_id covariate) j(group) string

**generate interaction terms between 2 other cutoff indicators and running variables.**
forvalues i=1/2{
    gen oth`i'_dist_cutoff=above_oth`i'*dist_cutoff
	gen oth`i'_dist2_cutoff=above_oth`i'*dist2_cutoff
	gen oth`i'_dist3_cutoff=above_oth`i'*dist3_cutoff	
}

**Pre-trim the sample to +/-6.**
keep if disc_samp_==1

**school-specific ik and parametric estimation.**
gen lambda=.
foreach sch in K89S X89S M89S{
    capture drop ik`sch'
    gen ik`sch'=0
	foreach var in female black hispanic frpl lep_yn{
	    capture drop `var'_above_cutoff `var'_above_oth1 `var'_above_oth2
	    gen `var'_above_cutoff = (covariate == "`var'")*above_cutoff
		gen `var'_above_oth1=(covariate == "`var'")*above_oth1
		gen `var'_above_oth2=(covariate == "`var'")*above_oth2
        **IK estimation for each type of exam schools.**
        local x dist_cutoff  
        qui rdob_mod2 y_ `x' if `x'!=.&covariate=="`var'"&group=="`sch'", c(0) dm(1)
        local h_opt=r(h_opt)
		*local h_opt=min(r(h_opt),3)
        **weight for local linear regression 
        qui replace lambda=(abs(`x')<=`h_opt')*(1-abs(`x')/`h_opt') if `x'!=.&covariate=="`var'"&group=="`sch'"
		qui replace ik`sch'=(lambda!=0&lambda!=.) if `x'!=.&covariate=="`var'"&group=="`sch'"
    }
	xi: reg y_ female_above_cutoff black_above_cutoff hispanic_above_cutoff frpl_above_cutoff lep_yn_above_cutoff  ///
	      i.covariate|dist_cutoff i.covariate|intdist_cutoff i.covariate*i.risk_set [aw = lambda] if group=="`sch'",cluster(osis_id)
	testparm *_above_cutoff	
	estadd scalar F_value=`r(F)'
    estadd scalar p_value=`r(p)'	
	estimates store ik`sch'
		
	xi: reg y_ female_above_cutoff black_above_cutoff hispanic_above_cutoff frpl_above_cutoff lep_yn_above_cutoff i.covariate|dist_cutoff /// 
	    i.covariate|dist2_cutoff i.covariate|dist3_cutoff i.covariate|intdist_cutoff i.covariate|intdist2_cutoff ///
		i.covariate|intdist3_cutoff i.covariate*i.risk_set if disc_samp_ == 1&group=="`sch'",cluster(osis_id)
	testparm *_above_cutoff
	estadd scalar F_value=`r(F)'
    estadd scalar p_value=`r(p)'
	estimates store para`sch'
}

**IK estimation for all exam schools.**
**get the stacked version of running variables and interactions.**
foreach x in dist_cutoff intdist_cutoff{
    capture drop ikK89S_`x' ikX89S_`x' ikM89S_`x'
    gen ikK89S_`x'=ikK89S*`x'
    gen ikX89S_`x'=ikX89S*`x'
    gen ikM89S_`x'=ikM89S*`x'
}
gen ikK89S_risk=ikK89S*risk_set
gen ikX89S_risk=ikX89S*risk_set
gen ikM89S_risk=ikM89S*risk_set
xi: reg y_ female_above_cutoff black_above_cutoff hispanic_above_cutoff frpl_above_cutoff lep_yn_above_cutoff i.covariate|ikX89S i.covariate|ikM89S ///
	i.covariate|ikK89S_dist_cutoff i.covariate|ikX89S_dist_cutoff i.covariate|ikM89S_dist_cutoff i.covariate|ikK89S_intdist_cutoff ///
    i.covariate|ikX89S_intdist_cutoff i.covariate|ikM89S_intdist_cutoff ///
	i.covariate*i.ikK89S_risk i.covariate*i.ikX89S_risk i.covariate*i.ikM89S_risk [aw = lambda],cluster(osis_id)
testparm *_above_cutoff
estadd scalar F_value=`r(F)'
estadd scalar p_value=`r(p)'
estimates store ik_all

**Parametric estimation for all exam schools.**
gen discK89S=(disc_samp_==1&group=="K89S")
gen discX89S=(disc_samp_==1&group=="X89S")
gen discM89S=(disc_samp_==1&group=="M89S")
**get the stacked version of running variables and interactions.**
foreach x of varlist dist*_cutoff intdist*_cutoff{
    capture drop discK89S_`x' discX89S_`x' discM89S_`x'
    gen discK89S_`x'=discK89S*`x'
    gen discX89S_`x'=discX89S*`x'
    gen discM89S_`x'=discM89S*`x'
}
gen discK89S_risk=discK89S*risk_set
gen discX89S_risk=discX89S*risk_set
gen discM89S_risk=discM89S*risk_set
xi: reg y_ female_above_cutoff black_above_cutoff hispanic_above_cutoff frpl_above_cutoff lep_yn_above_cutoff i.covariate|discX89S i.covariate|discM89S ///
	i.covariate|discK89S_dist_cutoff i.covariate|discX89S_dist_cutoff i.covariate|discM89S_dist_cutoff ///
	i.covariate|discK89S_dist2_cutoff i.covariate|discX89S_dist2_cutoff i.covariate|discM89S_dist2_cutoff ///
	i.covariate|discK89S_dist3_cutoff i.covariate|discX89S_dist3_cutoff i.covariate|discM89S_dist3_cutoff ///	
	i.covariate|discK89S_intdist_cutoff i.covariate|discX89S_intdist_cutoff i.covariate|discM89S_intdist_cutoff ///
	i.covariate|discK89S_intdist2_cutoff i.covariate|discX89S_intdist2_cutoff i.covariate|discM89S_intdist2_cutoff ///
	i.covariate|discK89S_intdist3_cutoff i.covariate|discX89S_intdist3_cutoff i.covariate|discM89S_intdist3_cutoff ///
	i.covariate*i.discK89S_risk i.covariate*i.discX89S_risk i.covariate*i.discM89S_risk if disc_samp_==1,cluster(osis_id)
testparm *_above_cutoff
estadd scalar F_value=`r(F)'
estadd scalar p_value=`r(p)'
estimates store para_all

**Output all regression results in a table.**
local rename1="female_above_cutoff female black_above_cutoff black hispanic_above_cutoff hispanic frpl_above_cutoff frpl lep_yn_above_cutoff lep_yn"
estout para* ik* using covbalance.txt,keep(female black hispanic frpl lep_yn) rename(`rename1') `options' replace  

***********************************************************************************************
**##########table 1, 7 and B1: School Choice, Complier Chars and Descriptive Stats.##########**
***********************************************************************************************
use regression_sample,clear
gen EnrollmentNYC=(grade9_inNYC==1&grade9_inexam==0)
keep osis_id appyr dist_cutoff* intdist_cutoff* Enrollment* above_cutoff* peer* num_teach ///
     per_valid_cert per_fewer_3yrs_exp per_mas_plus avg_class* ///
	 female black hispanic frpl lep_yn sped grade8_scoreSTM grade8_scoreELA appshs app_sample
duplicates drop
egen risk_set=group(appyr)
**Define the discontinuity samples.**
local disc_lb=-6
local disc_ub=6
capture drop disc_samp*
foreach sch in K89S X89S M89S{
    gen disc_samp_`sch'=(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub')
} 

xi i.appyr

local z="_Iappyr*"
#delimit ;
local schoolchars="peer_Math peer_English peer_blkhisp peer_frpl peer_fem
      avg_class_sizeENGLISH avg_class_sizeMATH per_valid_cert per_mas_plus 
	  per_fewer_3yrs_exp" ;
local complier_chars="female black hispanic frpl lep_yn sped grade8_scoreSTM grade8_scoreELA";
#delimit cr
matrix dest_all=J(4,1,.)
matrix dest_comp=J(4,1,.)
matrix table7=J(10,1,.)
matrix compchar = J(8, 1, .)

**applicants characteristics.**
* All NYC*
quietly: tabstat `complier_chars', save columns(stat)
mat b =  (r(StatTotal))
mat tableB1 = b'

keep if app_sample==1

foreach sch in K89S X89S M89S{
    local running_controls="dist_cutoff`sch' intdist_cutoff`sch'" 
	*School distribution on both sides of the cutoff.*
	mat column=(.,.)
    foreach j in NYC K89S X89S M89S{
	    capture rdob_mod2 Enrollment`j' dist_cutoff`sch' if disc_samp_`sch'==1,dm(1)
		local h=r(h_opt)
		if (`h'==.){
		    local h=5
		}
		capture drop w
	    gen w=(1-abs(dist_cutoff`sch'/`h'))*(abs(dist_cutoff`sch'/`h')<=1)
		capture ivreg2 Enrollment`j' above_cutoff`sch' `running_controls' `z' [aw=w] if disc_samp_`sch'==1,robust
        if (_rc==0){
		    mat column=(column\(_b[_cons],_b[_cons]+_b[above_cutoff`sch']))
		}
		else{
		    mat column=(column\(.,.))
		}
	}
	mat dest_all=(dest_all,column[2...,1...])
	
	*School distribution for compliers.*
	capture drop invd
	gen invd=1-Enrollment`sch'
	mat column=(.)
	foreach j in NYC K89S X89S M89S{
	    capture drop invd_`j'
		gen invd_`j'=invd*Enrollment`j'
		capture rdob_mod2 invd_`j' dist_cutoff`sch' if disc_samp_`sch'==1,dm(1)
		local h=r(h_opt)
		if (`h'==.){
		    local h=5
		}
		capture drop w
	    gen w=(1-abs(dist_cutoff`sch'/`h'))*(abs(dist_cutoff`sch'/`h')<=1)
		capture ivreg2 invd_`j' (invd = above_cutoff`sch') `running_controls' `z' [aw=w] if disc_samp_`sch'==1,robust
        if (_rc==0){
		    mat column=(column\_b[invd])
		}
		else{
		    mat column=(column\.)
		}
		drop invd_`j' w
	}
	mat dest_comp=(dest_comp,column[2...,1...])
	
	**school characteristics for compliers.**
	capture drop d 
	capture drop invd
	gen d=Enrollment`sch'
	gen invd=1-Enrollment`sch'
	mat column=(.,.)
    foreach j of local schoolchars {
	    mat row=(.)
		capture drop invd_`j'
		gen invd_`j'=invd*`j'
		capture rdob_mod2 invd_`j' dist_cutoff`sch' if disc_samp_`sch'==1,dm(1)
		local h=r(h_opt)
		if (`h'==.){
		    local h=5
		}
		capture drop w
	    gen w=(1-abs(dist_cutoff`sch'/`h'))*(abs(dist_cutoff`sch'/`h')<=1)
		capture ivreg2 invd_`j' (invd = above_cutoff`sch') `running_controls' `z' [aw=w] if disc_samp_`sch'==1,robust
        if (_rc==0){
		    mat row=(row,_b[invd])
		}
		else{
		    mat row=(row,.)
		}
		drop invd_`j' w
		
		capture drop d_`j'
		gen d_`j'=d*`j'
		capture rdob_mod2 d_`j' dist_cutoff`sch' if disc_samp_`sch'==1,dm(1)
		local h=r(h_opt)
		if (`h'==.){
		    local h=5
		}
		capture drop w
	    gen w=(1-abs(dist_cutoff`sch'/`h'))*(abs(dist_cutoff`sch'/`h')<=1)
		capture ivreg2 d_`j' (d = above_cutoff`sch') `running_controls' `z' [aw=w] if disc_samp_`sch'==1,robust
        if (_rc==0){
		    mat row=(row,_b[d])
		}
		else{
		    mat row=(row,.)
		}
		drop d_`j' w
		
		mat column=(column\row[1...,2...])		
    }
    mat table7=(table7,column[2...,1...])
    drop d invd	
	
	**applicants and enrollees characteristics.**
	* Applicants *
	quietly: tabstat `complier_chars' if regexm(appshs,"`sch'")==1, save columns(stat)
	mat b =(r(StatTotal))
	mat tableB1 =(tableB1, b')
		
	* Enrollees *
	quietly: tabstat `complier_chars' if Enrollment`sch'==1&regexm(appshs,"`sch'")==1, save columns(stat)
	mat b =(r(StatTotal))
	mat tableB1 =(tableB1, b')

	**complier characteristics.**
	mat column = (.)
	foreach j in `complier_chars' {	
	    capture drop d 
	    gen d=Enrollment`sch'
		capture: drop d_`j'
		gen d_`j' = d*`j'
		capture: rdob_mod2 d_`j' dist_cutoff`sch' if disc_samp_`sch'==1,dm(1)
		local h = r(h_opt)
		disp "`sch' - `j' - `h'"
		if (`h' == .) {
			local h = 5
		}
		capture: drop w
		gen w = (1 - abs(dist_cutoff`sch'/`h'))*(abs(dist_cutoff`sch'/`h') <= 1)
		capture noisily: ivreg2 d_`j' (d = above_cutoff`sch') `running_controls' `z' [aw=w] if disc_samp_`sch'==1,robust
		if (_rc == 0) {
			mat column = (column\ _b[d])
		}
		else {
			mat column = (column\ .)
		}
		drop d_`j' w
	}
	mat compchar = (compchar, column[2..., 1...])
	mat tableB1 =(tableB1, column[2..., 1...])
}
mat dest_all=dest_all[1...,2...]
mat dest_comp=dest_comp[1...,2...]
mat table1=(dest_all,dest_comp)
mat colnames table1="all_K89S0" "all_K89S1" "all_X89S0" "all_X89S1" "all_M89S0" "all_M89S1" "comp_K89S" "comp_X89S" "comp_M89S"
mat rownames table1="NYC" "K89S" "X89S" "M89S"

mat table7=table7[1...,2...]
mat colnames table7="comp_K89S0" "comp_K89S1" "comp_X89S0" "comp_X89S1" "comp_M89S0" "comp_M89S1"
mat rownames table7=`schoolchars'

mat rownames tableB1 = `complier_chars'
mat colnames tableB1 = "all_NYC" "app_K89S" "to_K89S" "comp_K89S" "app_X89S" "to_X89S" "comp_X89S" "app_M89S" "to_M89S" "comp_M89S"
		
mat2txt, matrix(table1) saving("table1.txt") replace
mat2txt, matrix(table7) saving("table7.txt") replace
mat2txt, matrix(tableB1) saving("tableB1.txt") replace

*********************************************************************************
**                                    Plots.                                   **
*********************************************************************************
cd /afs/econ.duke.edu/data/aa88/projects/nyc/results/AGS/new_codes
tempfile tmp
use regression_sample,clear
keep if app_sample==1
keep osis_id Offer* Enrollment* dist_cutoff* peer_Math peer_English peer_blkhisp ///
     std_mark coursecd ch* 
isid osis_id coursecd
replace coursecd="HXR" if coursecd=="HXR$"
reshape wide std_mark, i(osis_id) j(coursecd) string
ren std_markEXRL std_markEnglish
**Define the discontinuity samples.**
local disc_lb=-6
local disc_ub=6
foreach sch in K89S X89S M89S{
    gen disc_samp_`sch'=(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub')
}      
save `tmp',replace

local bin_min = `"-10"'
local bin_width = `"0.5"'
local bin_max = `"10"'
local xlabel = "-10(5)10"
**********************************************
**Part 1: Offer at Each (other) Exam School.**
**********************************************
foreach var in Offer{		
    local ylabel = "0(0.2)1"
    local title "`var' at Each Exam School for NYC Applicants"	 
    local ytitle="Offer Rate"
	local xtitle="Standardized Running Variable"
    foreach sch in K89S X89S M89S{
        if "`sch'" == "K89S" {
		  local subtitle = "Brooklyn Tech"
        }
	   else if "`sch'" == "X89S" {
		  local subtitle = "Bronx Science"
        }
	   else if "`sch'" == "M89S" {
		  local subtitle = "Stuyvesant"
        }
        local outcome `var'`sch'
        local x="dist_cutoff`sch'"
        preserve
        gen `var'_other=(`var'==1&`var'`sch'==0)
        egen temp_x=cut(`x') if `x' >= `bin_min' & `x' < `bin_max', at(`bin_min'(`bin_width')`bin_max') 
        bysort temp_x: egen temp_y = mean(`outcome')
        bysort temp_x: gen keep=_n
        capture rdob_mod2 `outcome' `x' if `outcome'!=.& `x'!=.&(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub'),dm(1)
        local h_opt=r(h_opt)
        if `h_opt' == . {
	       local h_opt = 1
        }
       capture rdob_mod2 `var'_other `x' if `var'_other != . & `x' != .&(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub'),dm(1)
	   local h_opt2 = r(h_opt)
	   if `h_opt2' == . {
		  local h_opt2 = 1
	   }
	   lpoly `outcome' `x' if `x' >=`bin_min' & `x' < 0,degree(1) kernel(triangle) bwidth(`h_opt') gen(xl sl) nograph
	   lpoly `outcome' `x' if `x' >= 0 & `x' <`bin_max',degree(1) kernel(triangle) bwidth(`h_opt') gen(xr sr) nograph
       lpoly `var'_other `x' if `x' >=`bin_min'&`x' < 0,degree(1) kernel(triangle) bwidth(`h_opt2') gen(xl_oth sl_oth) nograph
	   lpoly `var'_other `x' if `x' >= 0 &`x' <`bin_max',degree(1) kernel(triangle) bwidth(`h_opt2') gen(xr_oth sr_oth) nograph
	
	    *plots for variable at each school only.* 
		graph twoway (scatter temp_y temp_x if keep== 1 &temp_x >=`bin_min' & temp_x <`bin_max') ///
		(line sl xl if xl >=`bin_min' & xl < 0&sl>=0, lcolor(black)) ///
		(line sr xr if xr >= 0 & xr <`bin_max'&sr>=0, lcolor(black)), ///
        xlab(`xlabel') ylab(`ylabel') xline(0, lpattern(shortdash)) ytitle("") xtitle(" ") subtitle("`subtitle'") ///        
		legend(off) saving(temp_`sch', replace) ///
		
	   drop temp_x temp_y keep `var'_other
	   restore
    }
	*plots without legends.*
	graph combine temp_K89S.gph temp_X89S.gph temp_M89S.gph,cols(3) imargin(tiny) iscale(0.5) l1title(`ytitle', size(small)) /// 
	    b1title(`xtitle', size(small)) ycommon xcommon saving(`var'_each,replace)
	graph2tex, epsfile(`var'_each)
    erase temp_K89S.gph
    erase temp_X89S.gph
    erase temp_M89S.gph
    erase `var'_each.gph
}
*************************************************
**Part 2: Outcome variables at Any Exam School.**
*************************************************
foreach var of varlist Enrollment peer_Math peer_blkhisp std_markMathBG std_markEnglish{	
    if ("`var'" == "Enrollment") {
	    local ylabel = "0(0.2)1"
        local title "`var' at Any Exam School for NYC Applicants"
		local ytitle="Enrollment Rate"
	}
    if ("`var'"=="peer_Math"){
        local ylabel = "0(0.4)2"
        local test=substr("`var'",6,.)
        local title "Average Baseline `test' Score of Peers for NYC Applicants"  
        local ytitle="Peer Mean"
	}
	if ("`var'"=="peer_blkhisp"){
        local ylabel = "0(0.2)1"
        local title "Average Black and Hispanic Ratio of Peers for NYC Applicants"  
        local ytitle="Peer Proportion Black/Hispanic"
	}
    if substr("`var'",1,8)=="std_mark"{
        local ylabel = "0(0.4)2"
		local ytitle="Standardized Score"
        local test=substr("`var'",9,.)
        local pos=strpos("`test'","_")
        if `pos'!=0{
            local testname=substr("`test'",1,`pos'-1)+" "+substr("`test'",`pos'+1,.)
        }
        else{
            local testname="`test'"
        }
        if "`test'"=="MathBG"{
            local testname="Advanced Math"
        }
        local title "Average Regents `testname' Score for NYC Applicants"
     }
	 local xtitle="Standardized Running Variable"
    foreach sch in K89S X89S M89S{
        if "`sch'"=="K89S"{
           local subtitle = "Brooklyn Tech"
        }
	   if "`sch'" == "X89S" {
		  local subtitle = "Bronx Science"
        }
	   if "`sch'" == "M89S" {
		  local subtitle = "Stuyvesant"
        }
        preserve
        local outcome `var'
        local x="dist_cutoff`sch'"
        egen temp_x=cut(`x') if (`x' >= `bin_min' & `x' < `bin_max'), at(`bin_min'(`bin_width')`bin_max') 
        bysort temp_x: egen temp_y = mean(`outcome')
        bysort temp_x: gen keep=_n
        capture rdob_mod2 `outcome' `x' if `outcome'!=.& `x'!=.&(dist_cutoff`sch'>=`disc_lb'&dist_cutoff`sch'<=`disc_ub'),dm(1)
        local h_opt=r(h_opt)
        disp "Optimal Bandwidth for `var' of `sch': `h_opt'"
        qui count if `x'>=-`h_opt'&`x'<=`h_opt'
        disp "Number of students in bandwidth for `var' of `sch': `r(N)'"

        if (`h_opt' == .){
	       local h_opt = 1
        }
		lpoly `outcome' `x' if `x' >=`bin_min' & `x' < 0,degree(1) kernel(triangle) bwidth(`h_opt') gen(xsl ysl) nograph
		lpoly `outcome' `x' if `x' >= 0 & `x' <`bin_max',degree(1) kernel(triangle) bwidth(`h_opt') gen(xsr ysr) nograph
		
        graph twoway (scatter temp_y temp_x if keep== 1 & (temp_x >=`bin_min' & temp_x <`bin_max')) ///
		(line ysl xsl if xsl >=`bin_min' & xsl < 0&ysl>=0, lcolor(black)) ///
		(line ysr xsr if xsr >= 0 & xsr <`bin_max'&ysr>=0, lcolor(black)), ///
		xlab(`xlabel') ylab(`ylabel') xline(0, lpattern(shortdash)) ytitle("") xtitle(" ") subtitle(`subtitle') legend(off) nodraw
        graph save temp_`sch',replace
        drop temp_x temp_y keep
        restore
    }
    graph combine temp_K89S.gph temp_X89S.gph temp_M89S.gph,cols(3) imargin(tiny) iscale(0.5) l1title(`ytitle', size(small)) /// 
	    b1title(`xtitle', size(small)) ycommon xcommon saving(`var',replace)
	graph2tex,epsfile(`var')
    erase temp_K89S.gph
    erase temp_X89S.gph
    erase temp_M89S.gph
    erase `var'.gph
}

log close
exit



